/******************************************
GSM-Routines to communicate with nokia Phone
AUTHOR: Roland Praml <praml.roland@t-online.de> www.praml.net
*****************************************/

#include <io.h>
#include "mbus.h"
#include <interrupt.h>
#include <sig-avr.h>
#include "gsm.h"
#include "lcd.h"


u08 GSM_command(u08 cmd, u08 *buf, u08 size){
  u08 i = 3;
  u16 d;
  while (i--) {
    for (d=0; d < 6000; d++) asm("nop; nop; nop;");
    if (MBUS_command(cmd, buf, size)) return 1;

  }
  GSM_connected = 0;
  return 0;
}


/* Read clock from nokia */
void GSM_clock(void) {
  u08 buf[4]= {0x00, 0x01, 0x00, 0x62};

  if (GSM_command(0x11, buf, 0x04)) {
    GSM_year = ((u16)(MBUS_buffer[8] << 8)  + MBUS_buffer[9]);
    GSM_month = MBUS_buffer[10];
    GSM_day = MBUS_buffer[11];
    GSM_hour = MBUS_buffer[12];
    GSM_minute = MBUS_buffer[13];
    GSM_second = MBUS_buffer[14];
  }
}

/* Link with the phone */
void GSM_link(void) {
  u08 buf[1]= {0x04};
  u16 d;
  for (d=0; d < 6000; d++) asm("nop; nop; nop;");
  MBUS_send_frame(0xD0, buf, 0x01);
  if (MBUS_buffer[0] == 0x05) GSM_connected = 1;
}



/* read Network status from nokia */
void GSM_network_status(void) {
  u08 buf[4]= {0x00, 0x01, 0x00, 0x01};
  if (GSM_command(0x04, buf, 0x04)) {
    GSM_mode = MBUS_buffer[4];
    GSM_signal = MBUS_buffer[5];
    //GSM_pwr = MBUS_buffer[7];
    GSM_batt = MBUS_buffer[8];
  } else GSM_mode = 255;

  /*  alarm = 0;
  if (GSM_mode != 1) alarm = 2;
  if (GSM_mode == 2) alarm = 1;
  if (GSM_pwr != 1) alarm++;
  */

}


u08 buf[] = { 0x00,  0x01,  0x00,  0x01,  // HEADER
	      0x02,  0x00, // Trailer
	      
	      // Octet pack of SMSC
	      0x07, 0x91,  0x94,  0x71,  0x01,  0x67,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  // SMSC
	      //+              49     17     10     76     00     00
	      
	      0x15,  
	      0x00,  
	      0x00,  
	      0x00,  //<-- CLASS (0xF0 for immediate show)
	      0x20,  //Length of message
	      //hardcode till here
	      
	      //Encode number (semi octet)... 0x81 = unknown type
	      0x00, 0x00, 0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  
	      //88     88     05
	      // offset 23
	      
	      
	      // validity
	      0xA7,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  
		
	      // Payload
	      
	      // Message Text (Ofs 42)
	      0xB0,  0x98,  0x6C,  0x46,  0xAB,  0xD9,  0x6E,  
	      0xB8,  0x5C,  0x50,  0x38,  0x24,  0x16,  0x8D,
	      
	      // additional Text (date & time)
	      0x20,  0x85,  0x62,  0xDA,  0x9C,  0xB6,  0x82,  
	      0xEC,  0xB0,  0xBC,  0xDD,  0x2E,  0xB3,  0xC9};


u08 SemiOctetPack(u08 *Number, u08 *Output);

u08 Pack7Bit(u08 *input, u08 *output, u08 in_len);
/* Send a SMS */ 
u08 GSM_sms (u08 *number, u08 *message) {


  u16 d;
  u08 i = 3;
  //u16 d;
  lcd_clrscr();
  lcd_puts(number);
  buf[23]=SemiOctetPack(number, &buf[24]);
  if (buf[23] < 4) return;
  Pack7Bit(message, &buf[42], 16);
  SCRATCH[0]=' ';
  SCRATCH[1]='0' + GSM_day / 10;
  SCRATCH[2]='0' + GSM_day % 10;
  SCRATCH[3]='.';
  SCRATCH[4]='0' + GSM_month / 10;
  SCRATCH[5]='0' + GSM_month % 10;
  SCRATCH[6]='.';
  SCRATCH[7]='0' + (GSM_year - 2000) / 10;
  SCRATCH[8]='0' + (GSM_year - 2000) % 10;
  SCRATCH[9]=' ';
  SCRATCH[10]='0' + GSM_hour / 10;
  SCRATCH[11]='0' + GSM_hour % 10;
  SCRATCH[12]=':';
  SCRATCH[13]='0' + GSM_minute / 10;
  SCRATCH[14]='0' + GSM_minute % 10;
  SCRATCH[15]=' ';
  SCRATCH[16]=0;
  Pack7Bit(SCRATCH, &buf[56], 16);
  /*
    0123456701234567012345670123456701234567
    Einsatzmeldung   TT.MM.JJ HH:MM 
  */
  //GSM_state=0; // Begin

  for (d=0; d < 50000; d++) asm ("nop");
  if (GSM_command(0x02, buf, sizeof(buf))) {
    lcd_gotoxy(0,1);
    lcd_puts("Sende ");
    lcd_gotoxy(0,6);
    i = 8;
    // wait until sms sent
    while (--i) {
      lcd_putc('.');
      //GSM_state++;
      if (MBUS_recv_frame(0x02)) {
	if ( (MBUS_buffer[3] == 0x02) && 
	     //     (MBUS_buffer[4] == 0x64) && 
	     //  (MBUS_buffer[5] == 0x92) && 
	     (MBUS_buffer[6] == 0x00)) break; // SMS sent, denke ich
      }
    }
    
    
    if (i) {
      lcd_gotoxy(0,1);
      lcd_puts("Erfolg ");
      for (d=0; d < 50000; d++) asm ("nop");
      return 1;
    } else {
      lcd_gotoxy(0,1);
      lcd_puts("Timeout ");
      for (d=0; d < 50000; d++) asm ("nop");
      return 0;
    }
  }
  lcd_puts("Fehler :-( ");
  for (d=0; d < 50000; d++) asm ("nop");
  return 0;
}

void GSM_init(void) {
  MBUS_init();
}


u08 SemiOctetPack(u08 *Number, u08 *Output) {
  u08 *IN_NUM = Number;  /* Pointer to the input number */
  u08 *OUT_NUM = Output; /* Pointer to the output */
  u08 count = 0; /* This variable is used to notify us about count of already
		    packed numbers. */

  /* The first byte in the Semi-octet representation of the address field is
     the Type-of-Address. This field is described in the official GSM
     specification 03.40 version 6.1.0, section 9.1.2.5, page 33. We support
     only international and unknown number. */
  
  *OUT_NUM++ = 0x81;
  
  /* The next field is the number. It is in semi-octet representation - see
     GSM scpecification 03.40 version 6.1.0, section 9.1.2.3, page 31. */
  while (*IN_NUM  ) {
    while(*IN_NUM < '0' || *IN_NUM > '9') {
      IN_NUM++;
      if (IN_NUM - Number > 16) return 0;
    }
    if (count & 0x01) {
      *OUT_NUM = *OUT_NUM | ((*IN_NUM - '0') << 4);
      OUT_NUM++;
    }
    else
      *OUT_NUM = *IN_NUM - '0';
    count++; IN_NUM++;
  }
  /* We should also fill in the most significant bits of the last byte with
           0x0f (1111 binary) if the number is represented with odd number of
           digits. */
  if (count & 0x01) {
    *OUT_NUM = *OUT_NUM | 0xf0;
    OUT_NUM++;
  }
  return (2 * (OUT_NUM - Output - 1) - (count % 2));
}

u08 Pack7Bit(u08 *input, u08 *output, u08 in_len) {

  u08 *OUT_NUM = output; /* Current pointer to the output buffer */
  u08 *IN_NUM = input;  /* Current pointer to the input buffer */
  u08 Bits;                    /* Number of bits directly copied to
				  the output buffer */

  Bits = 7;
   
  while ((IN_NUM - input) < in_len) {
    *OUT_NUM = *IN_NUM >> (7 - Bits);
    /* If we don't write at 0th bit of the octet, we should write
       a second part of the previous octet */
    if (Bits != 7)
      *(OUT_NUM-1) |= (*IN_NUM & ((1 << (7-Bits)) - 1)) << (Bits+1);
    if (Bits == 0) {
      Bits = 8;
    } else OUT_NUM++;
    Bits--;
    IN_NUM++;
  }
  return (OUT_NUM - output);
}
